"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
var utils_1 = require("./utils");
var REGEX_PARAMS = /([:*])(\w+)/g;
var REGEX_REPLACE_VAR = '([^/]+)';
function onRouterChange(fn) {
    window.addEventListener('hashchange', function () {
        fn();
    });
}
function go(path) {
    window.location.hash = '#' + path;
}
function useRouter(routesArray) {
    var params = Object.create(null);
    var routes = Object.create(null);
    var currentComponent = null;
    for (var _i = 0, routesArray_1 = routesArray; _i < routesArray_1.length; _i++) {
        var r = routesArray_1[_i];
        r.path = utils_1.cleanPath(r.path);
        routes[r.path] = r.component;
    }
    function getPath() {
        return utils_1.cleanPath(location.hash);
    }
    function regexToParams(match, names) {
        if (names.length === 0)
            return null;
        if (!match)
            return null;
        // filter params, skip full match string -> params mapping
        var matchInfo = match.slice(1, match.length);
        return matchInfo.reduce(function (paramObj, value, index) {
            var name = names[index];
            paramObj[name] = value;
            return paramObj;
        }, {});
    }
    function createParamsPattern(routerPath) {
        // posts/:id -> posts/([^/]+)
        var paramNames = [];
        var pattern = routerPath.replace(REGEX_PARAMS, function (_1, _2, name) {
            paramNames.push(name);
            return REGEX_REPLACE_VAR;
        });
        return { paramNames: paramNames, pattern: pattern };
    }
    function match(browserPath) {
        if (utils_1.isObject(routes[browserPath])) {
            currentComponent = routes[browserPath];
            return routes[browserPath];
        }
        // Assign url params
        params = Object.create(null);
        for (var _i = 0, _a = Object.keys(routes); _i < _a.length; _i++) {
            var routerPath = _a[_i];
            // browserPath = 'posts/37739';
            // routerPath = 'posts/:id'
            if (utils_1.includes(routerPath, ':')) {
                var _b = createParamsPattern(routerPath), pattern = _b.pattern, paramNames = _b.paramNames;
                var regexp = new RegExp(pattern);
                var match_1 = browserPath.match(regexp);
                if (match_1) {
                    params = regexToParams(match_1, paramNames);
                    currentComponent = routes[routerPath];
                    return routes[routerPath];
                }
            }
        }
        // Handle 404
        if (utils_1.isObject(routes['*'])) {
            currentComponent = routes['*'];
            return routes[browserPath];
        }
        var default404Route = {
            render: function () { return "Not found"; }
        };
        currentComponent = default404Route;
        return default404Route;
    }
    function routerContextFn() {
        return {
            params: function () { return params; },
            go: go
        };
    }
    function renderer(render, forceUnmount) {
        onRouterChange(function () {
            if (currentComponent) {
                forceUnmount(currentComponent);
            }
            render(match(getPath()));
        });
        render(match(getPath()));
    }
    return {
        renderer: renderer,
        routerContextFn: routerContextFn
    };
}
exports.createRouter = useRouter;
exports.default = {
    createRouter: exports.createRouter
};
